/* ----------------------------------------------------
   CONSTANTS
------------------------------------------------------*/

// --- GLOBALS --- //

// Version numbers

var VERSION_MAJOR = '3';
var VERSION_MINOR = '1';
var VERSION = VERSION_MAJOR + '.' + VERSION_MINOR;

// Paths

var PATH_SERVICE         = '../../../service';
var PATH_SERVICE_FLAGS   = PATH_SERVICE + '/flags';
var PATH_SERVICE_JS      = PATH_SERVICE + '/js';
var PATH_SERVICE_IMAGES  = PATH_SERVICE + '/images';
var PATH_SERVICE_INSTALL = PATH_SERVICE + '/install';
var PATH_SERVICE_LANG    = PATH_SERVICE_JS + '/languages';
var PATH_LANG            = '/languages';
var PATH_SERVICE_THUMBS  = PATH_SERVICE + '/thumbs';

// Filenames

var FILENAME_THUMB       = '52.jpg';
var FILENAME_BOOK_ICON   = 'link_book.png';
var FILENAME_WEB_ICON    = 'link_web.png';

// Links

var LINK_EMAIL       = 'mozaik3d@mozaik.info.hu';
var LINK_INSTALL_EXE = PATH_SERVICE_INSTALL + '/UnityWebPlayer.exe';
var LINK_MOZAWEB     = 'http://mozaweb.hu/mozaik3D';


// --- PLUGIN --- //


// Error codes & messages

var ERROR_NONE       = 0;
var ERROR_BADOS      = 1;
var ERROR_BADBROWSER = 2;
var ERROR_NOPLUGIN   = 3;
var ERROR_UNKNOWN    = 4;
var ERROR_CANNOTLOAD = 5;

var errorMessages = new Array();

function ERROR_MESSAGES(errorCode)
{
	errorMessages[ERROR_NONE]       = '<p class="error">' + LANG_ERROR_NONE + '</p>';
	errorMessages[ERROR_BADOS]      = '<p class="error">' + LANG_ERROR_BADOS + '</p>';
	errorMessages[ERROR_BADBROWSER] = '<p class="error">' + LANG_ERROR_BADBROWSER + '</p>';
	errorMessages[ERROR_NOPLUGIN]   = '<p class="error">' + LANG_ERROR_NOPLUGIN + '</p>';
	errorMessages[ERROR_UNKNOWN]    = '<p class="error">' + LANG_ERROR_UNKNOWN + '</p>';
	errorMessages[ERROR_CANNOTLOAD] = '<p class="error">' + LANG_ERROR_CANNOTLOAD + '</p>';

	// Put plugin link into noplugin message
	errorMessages[ERROR_NOPLUGIN] = errorMessages[ERROR_NOPLUGIN].replace(/%pluginlink%/g, '<a href="' + LINK_INSTALL_EXE + '">' + LANG_ERROR_PLUGINLINK + '</a>');

	return errorMessages[errorCode];
}

// --- VIEWER --- //

// Default sizes

var CONTAINER_WIDTH      = 980;
var CONTAINER_HEIGHT     = 610;

var CONTENT3D_WIDTH      = 962;
var CONTENT3D_HEIGHT     = 525;

var WINDOW_WIDTH         = CONTAINER_WIDTH;
var WINDOW_HEIGHT        = CONTAINER_HEIGHT;

var WRAPPER_WIDTH        = CONTAINER_WIDTH;

var CONTENTS_HEIGHT      = CONTENT3D_HEIGHT + 5;
var SIDEBAR_HEIGHT       = CONTENT3D_HEIGHT - 10;  // padding -10px;
var SIDEBAR_WIDTH        = 244;
var TOPBAR_HEIGHT        = 35;
var BOTTOMBAR_HEIGHT     = 45;

var TOOLBUTTON_WIDTH     = 29;
var TOOLBUTTON_HEIGHT    = 29;
var CONTROLBUTTON_HEIGHT = 34;

// Color

var CONTENT3D_BGCOLOR = '#576e9c';

// Slider defaults

var SLIDER_VALUE  = 50;
var SLIDER_MIN    = 1;
var SLIDER_MAX    = 100;

// Html (constants packed into functions)

function HTML_TOOLBUTTONS()
{
	var text =
		'<div id="button_sound" class="button" onclick="viewer.buttonSoundClick(this);" title="' + LANG_TOOLBUTTON_SOUND + '"></div>' +
    	'<div id="volumeslider"></div>' +
		'<div id="button_pan" class="button" onclick="viewer.buttonPanClick(this);" title="' + LANG_TOOLBUTTON_PAN + '"></div>' +
		'<div id="button_rotating" class="button" onclick="viewer.buttonRotatingClick(this);" title="' + LANG_TOOLBUTTON_ROTATING + '"></div>' +
		'<div id="button_titles" class="button" onclick="viewer.buttonTitlesClick(this);" title="' + LANG_TOOLBUTTON_TITLES + '"></div>' +
		'<div id="button_anaglyph" class="button" onclick="viewer.buttonAnaglyphClick(this);" title="' + LANG_TOOLBUTTON_ANAGLYPH + '"></div>' +
		'<div id="button_info" class="button" onclick="viewer.buttonInfoClick(this);" title="' + LANG_TOOLBUTTON_INFO + '"></div>' +
		'<div id="button_help" class="button" onclick="viewer.buttonHelpClick(this);" title="' + LANG_TOOLBUTTON_HELP + '"></div>' +
		'<div id="button_links" class="button" onclick="viewer.buttonLinksClick(this);" title="' + LANG_TOOLBUTTON_LINKS + '"></div>' +
		'<div id="button_options" class="button" onclick="viewer.buttonOptionsClick(this);" title="' + LANG_TOOLBUTTON_OPTIONS + '"></div>' +
		'<div id="button_fullscreen" class="button" onclick="viewer.buttonFullscreenClick(this);" title="' + LANG_TOOLBUTTON_FULLSCREEN + '"></div>';

	return text;
}

function HTML_SCROLLBUTTONS()
{
	var text =
		'<div id="button-scroll-left" onmousedown="viewer.scrollButtonLeftClick();" onmouseout="viewer.stopScrollTimers();" onmouseup="viewer.stopScrollTimers();"></div>' +
		'<div id="button-scroll-right" onmousedown="viewer.scrollButtonRightClick();" onmouseout="viewer.stopScrollTimers();" onmouseup="viewer.stopScrollTimers();"></div>';
	return text;
}

function HTML_HELPTEXT()
{
	var text =
		'<h2 class="center" style="margin: 8px 0 10px 0;">' + LANG_HELP_TITLE + '</h2>' +
		'<p class="center">' + LANG_HELP_INSTALL_INTRUCTIONS + '</p>' +
		'<hr />' +
		'<h3 class="center">' + LANG_HELP_MOUSE_TITLE + '</h3>' +
		'<p><img class="mousecontrol" src="' + PATH_SERVICE + '/images/help_mouse_orbit.png" alt="" /><strong>' + LANG_HELP_MOUSE_ROTATE_TITLE + '</strong> ' + LANG_HELP_MOUSE_ROTATE + '</p>' +
		'<p><img class="mousecontrol" src="' + PATH_SERVICE + '/images/help_mouse_zoom.png" alt="" /><strong>' + LANG_HELP_MOUSE_ZOOM_TITLE + '</strong> ' + LANG_HELP_MOUSE_ZOOM + '</p>' +
		'<p><img class="mousecontrol" src="' + PATH_SERVICE + '/images/help_mouse_pan.png" alt="" /><strong>' + LANG_HELP_MOUSE_PAN_TITLE + '</strong> ' + LANG_HELP_MOUSE_PAN + '</p>' +
		'<p><strong>'+ LANG_HELP_WALK_TITLE +'</strong> ' + LANG_HELP_WALK + '</p>' +
		'<hr />' +
		'<h3 class="center">' + LANG_HELP_BUTTONS_TITLE + '</h3>' +
		'<p><img class="buttoncontrol" src="' + PATH_SERVICE + '/images/help_button_pan.png" alt="" />' + LANG_HELP_BUTTONS_PAN + '</p>' +
		'<p><img class="buttoncontrol" src="' + PATH_SERVICE + '/images/help_button_sound.png" alt="" />' + LANG_HELP_BUTTONS_SOUND + '</p>' +
		'<p><img class="buttoncontrol" src="' + PATH_SERVICE + '/images/help_button_rotating.png" alt="" />' + LANG_HELP_BUTTONS_ROTATING + '</p>' +
		'<p><img class="buttoncontrol" src="' + PATH_SERVICE + '/images/help_button_titles.png" alt="" />' + LANG_HELP_BUTTONS_TITLES + '</p>' +
		'<p><img class="buttoncontrol" src="' + PATH_SERVICE + '/images/help_button_anaglyph.png" alt="" />' + LANG_HELP_BUTTONS_ANAGLYPH + '</p>' +
		'<p><img class="buttoncontrol" src="' + PATH_SERVICE + '/images/help_button_info.png" alt="" />' + LANG_HELP_BUTTONS_INFO + '</p>' +
		'<p><img class="buttoncontrol" src="' + PATH_SERVICE + '/images/help_button_links.png" alt="" />' + LANG_HELP_BUTTONS_LINKS + '</p>' +
		'<p><img class="buttoncontrol" src="' + PATH_SERVICE + '/images/help_button_help.png" alt="" />' + LANG_HELP_BUTTONS_HELP + '</p>' +
		'<p><img class="buttoncontrol" src="' + PATH_SERVICE + '/images/help_button_options.png" alt="" />' + LANG_HELP_BUTTONS_OPTIONS + '</p>' +
		'<hr />' +
		'<h3 class="center">' + LANG_HELP_CONTROLS_TITLE + '</h3>' +
		'<p class="center">'+ LANG_HELP_CONTROLS +'</p>' +
		'<hr />' +
		'<h3 class="center">' + LANG_HELP_DEV_TITLE + '</h3>' +
		'<p class="center"><strong>' + LANG_HELP_DEV_PUBLISHER + '</strong><br />' + LANG_HELP_DEV_PUBLISHER_ADDR + '</p>' +
		'<p class="center"><strong>' + LANG_HELP_DEV_DEVELOPER + '</strong><br />' + LANG_HELP_DEV_DEVELOPER_ADDR + '</p>' +
		'<hr />' +
		'<p class="center">' + LANG_HELP_DEV_REFLECTIONS + '</p>' +
		'<p class="center">' + LANG_HELP_DEV_LEGAL + '</p>' +
		'<p class="center">' + LANG_HELP_DEV_VERSION_VIEWER + '<br />' + LANG_HELP_DEV_VERSION_3D + '<br /><br /></p>';

	// Replace %parameter% strings
	text = text.replace(/%pluginlink%/g, '<a href="' + LINK_INSTALL_EXE + '">' + LANG_HELP_INSTALL_PLUGINLINK + '</a>');
	text = text.replace(/%emaillink%/g, '<a href="mailto:' + LINK_EMAIL + '?subject=' + title + '">' + LINK_EMAIL + '</a>');
	text = text.replace(/%progver%/g, VERSION);
	text = text.replace(/%3dtitle%/g, title);
	text = text.replace(/%3dver%/g, templateVersion);

	return text;
}

function HTML_OPTIONSTEXT()
{
	var text =
		'<h2 class="center" style="margin: 8px 0 10px 0;">' + LANG_OPTIONS_TITLE + '</h2>' +
		'<hr />' +
		'<p style="margin: 5px 0 5px 5px;"><strong>' + LANG_OPTIONS_DETAIL + '</strong></p>' +
		'<p style="margin: 5px 0 5px 5px;">' + LANG_OPTIONS_DETAIL_TEXT + '</p>' +
		'<p class="checkbox"><span id="setting_detail_low" onclick="viewer.settingDetailClick(this);">' + LANG_OPTIONS_DETAIL_LOW + '</span></p>' +
		'<p class="checkbox"><span id="setting_detail_medium" onclick="viewer.settingDetailClick(this);">' + LANG_OPTIONS_DETAIL_MEDIUM + '</span></p>' +
		'<p class="checkbox"><span id="setting_detail_high" onclick="viewer.settingDetailClick(this);">' + LANG_OPTIONS_DETAIL_HIGH + '</span></p>' +
		'<hr />';

	if (typeof languages != 'undefined') {
		text +=
		'<p style="margin: 5px 0 5px 5px;"><strong>' + LANG_OPTIONS_LANGUAGES + '</strong></p>' +
		'<p style="margin: 5px 0 5px 5px;">' + LANG_OPTIONS_LANGUAGES_TEXT + '</p>';

		// Add language checkboxes & flag images (as backgrounds)
		for (var i = 0; i < languages.length; i++) {
			var languageID = 'setting_languages_' + languages[i].toLowerCase();
			text +=	'<p class="checkbox" style="background: url(' + PATH_SERVICE_FLAGS + '/' + languages[i] + '.png) 20px 3px no-repeat;"><span class="flag" id="' + languageID + '" onclick="viewer.settingLanguagesClick(this);">' + getLanguageTitle(languages[i]) + '</span></p>';
		}

		text +=
		'<hr />';
	}

	return text;
}

function HTML_LINKSTEXT()
{
	var text=
		'<h2 class="center" style="margin: 8px 0 10px 0;">' + LANG_LINKS_TITLE + '</h2>' +
		'<hr style="margin-bottom: 18px" />';

	if (typeof links != 'undefined') {
		for (var i = 0; i < links.length; i++) {
			text += '<div class="links">';
			text += '<p>';
			text += '<a href="#nogo" onclick="viewer.linksClick(' + i + ')">';
			text += '<img class="thumb" src="' + PATH_SERVICE_THUMBS + '/' + links[i].id + '/' + FILENAME_THUMB + '" alt="" />';
			// text += '<img class="icon" src="' + PATH_SERVICE_IMAGES + '/' + (links[i].offline ? FILENAME_BOOK_ICON : FILENAME_WEB_ICON) + '" alt="" />';
			text += links[i].title;
			text += '</a>';
			text += '</p>';
			text += '</div>';
		}
	}

	text +=
	   	'<hr style="margin-top: 18px" />';

	return text;
}


// InfoBar special texts

var IT_CLEAR   = '%clear%';
var IT_HELP    = '%help%';
var IT_OPTIONS = '%options%';
var IT_LINKS   = '%links%';

var IT_DEFAULT_TEXT_1 = 'gombhoz tartozó szöveg';
var IT_DEFAULT_TEXT_2 = 'Gomb leírása';
var IT_DEFAULT_TEXT_3 = '_text';


// Window resize modes

var RM_INTERNAL = 0;
var RM_EXTERNAL = 1;

// Window states (for internal resizing)

var WS_NORMAL    = 0;
var WS_MAXIMIZED = 1;

// Button types

var BT_NONE          = 0;
var BT_TOOLBUTTON    = 1;
var BT_CONTROLBUTTON = 2;

// Hidden DIV IDs

var HIDDEN_SETTINGS_ID = 'settings';

// Sound volume slider

var VOLUME_MIN = 1;
var VOLUME_MAX = 10;

// Language

var LANGUAGE_DEFAULT = 'hu';
var LANGUAGE_TITLES = [
	{
		id    : 'hu',
		title : 'Magyar'
	},
	{
		id    : 'en',
		title : 'English'
	}
];


/* ----------------------------------------------------
   GLOBAL VARIABLES
------------------------------------------------------*/

var viewer = new Viewer();
var internals = new Internals();
var jsLoader = new JSLoader();
var cssLoader = new CSSLoader();

var HTMLVersion;
var URLParams = new Array();


/* ----------------------------------------------------
   GLOBAL FUNCTIONS & MISC UTILS
------------------------------------------------------*/

function element(id)
{
	return document.getElementById(id);
}

function addEvent(obj, evType, fn)
{
	if (obj.addEventListener) {
		obj.addEventListener(evType, fn, false);
		return true;
	} else if (obj.attachEvent) {
		var r = obj.attachEvent("on"+evType, fn);
		return r;
	} else {
		return false;
	}
}

function removeEvent(obj, evType, fn)
{
	if (obj.removeEventListener) {
		obj.removeEventListener(evType, fn, false);
		return true;
	} else if (obj.detachEvent) {
		var r = obj.detachEvent("on"+evType, fn);
		return r;
	} else {
		return false;
	}
}

function isStrInClassName(className, str)
{
	var si = -1;
	if (className)
		si = className.indexOf(str);
	return si > -1;
}

function removeChildNodes(ctrl)
{
	while (ctrl.childNodes[0]) {
		ctrl.removeChild(ctrl.childNodes[0]);
	}
}

function doNothing()
{
	return false;
}

function disableSelection(id)
{
	if (id == 'all') {
		var obj = document;
	} else {
		var obj = element(id);
	}

	if (document.all) {  
		obj.onselectstart = doNothing; // Explorer
	} else {             
		obj.onmousedown = doNothing;   // Firefox
	}
}

function getWindowInnerSize()
{
	var w, h;
	if (self.innerHeight) {  // all except Explorer
		w = self.innerWidth;
		h = self.innerHeight;
	} else if (document.documentElement && document.documentElement.clientHeight) {   // Explorer 6 Strict Mode
		w = document.documentElement.clientWidth;
		h = document.documentElement.clientHeight;
	} else if (document.body) {   // other Explorers
		w = document.body.clientWidth;
		h = document.body.clientHeight;
	}
	return {width: w, height: h};
}


function removeHashmark(str) {
	if (str.indexOf('#') == 0)
	    str = str.replace('#', '');
	return str;
}

function autoEllipseText(element, text, width)
{
	element.innerHTML = '<span id="ellipsisSpan" style = "white-space:nowrap;">' + text + '</span>';
	var inSpan = document.getElementById('ellipsisSpan');
	if (inSpan.offsetWidth > width) {
		var i = 1;
		inSpan.innerHTML = '';
		while (inSpan.offsetWidth < width && i < text.length) {
			inSpan.innerHTML = text.substr(0, i) + '&hellip;';
			i++;
		}
		var returnText = inSpan.innerHTML;
		element.innerHTML = '';
		return returnText;
	}
	return text;
}

function clampValue0(value) {
	return value < 0 ? 0 : value;
}

function clampValue1(value) {
	return value < 1 ? 1 : value;
}

function variableExists(variableName) {
	var s = '!(typeof ' + variableName + ' == "undefined")';
	return eval(s);
}

String.prototype.trim = function()     // prototyping is cool
{
	return this.replace(/^\s\s*/, '').replace(/\s\s*$/, '');
}

Array.prototype.indexOf = function(obj) {
	for (var i = 0; i < this.length; i++) {
		if (this[i] == obj)
			return i;
	}
	return -1;
}

Array.prototype.contains = function(obj) {
	return this.indexOf(obj) >= 0;
}

function getHTMLVersion()
{
	var majorVersion = 0;
	var minorVersion = 0;

	// gets html version number from body tag's id (example: id="service_3_1" returns major 3 and minor 1 version number)
	if (document.body && document.body.id) {
		var s = document.body.id;
		if (s.indexOf('service_') == 0) {
			s = s.replace('service_', '');
			var a = s.split('_');
			if (a.length == 2) {
				if (!isNaN(parseInt(a[0])))
					majorVersion = parseInt(a[0]);
				if (!isNaN(parseInt(a[1])))
					minorVersion = parseInt(a[1]);
			}
		}
	}
	return { major : majorVersion, minor : minorVersion };
}

function languageAvailable(languageID)
{
	var r = false;
	languageID = languageID.trim().toLowerCase();
	if (typeof languages != 'undefined') {
		for (var i = 0; i < languages.length; i++) {
			if (languages[i].trim().toLowerCase() == languageID) {
				r = true;
				break;
			}
		}
	}
	return r;
}

function getLanguageTitle(languageID)
{
	var r = '';
	languageID = languageID.trim().toLowerCase();
	if (typeof languages != 'undefined') {
		for (var i = 0; i < LANGUAGE_TITLES.length; i++) {
			if (LANGUAGE_TITLES[i].id.trim().toLowerCase() == languageID) {
				r = LANGUAGE_TITLES[i].title;
				break;
			}
		}
	}
	return r;
}


/* ----------------------------------------------------
	BROWSER DETECT OBJECT (from: www.quirksmode.org)
-------------------------------------------------------*/

var browserDetect = {
	init: function()
	{
		this.browser = this.searchString(this.dataBrowser) || "An unknown browser";
		this.version = this.searchVersion(navigator.userAgent) || this.searchVersion(navigator.appVersion) || "an unknown version";
		this.OS = this.searchString(this.dataOS) || "an unknown OS";
	},

	searchString: function(data)
	{
		for (var i = 0; i < data.length; i++)	{
			var dataString = data[i].string;
			var dataProp = data[i].prop;
			this.versionSearchString = data[i].versionSearch || data[i].identity;
			if (dataString) {
				if (dataString.indexOf(data[i].subString) != -1)
					return data[i].identity;
			} else if (dataProp) {
				return data[i].identity;
			}
		}
	},

	searchVersion: function(dataString)
	{
		var index = dataString.indexOf(this.versionSearchString);
		if (index == -1)
			return;
		return parseFloat(dataString.substring(index + this.versionSearchString.length + 1));
	},

	dataBrowser: [
		{ string: navigator.userAgent, subString: "Chrome", identity: "Chrome" },
		{ string: navigator.userAgent, subString: "OmniWeb", versionSearch: "OmniWeb/", identity: "OmniWeb" },
		{ string: navigator.vendor, subString: "Apple", identity: "Safari", versionSearch: "Version" },
		{ prop: window.opera, identity: "Opera" },
		{ string: navigator.vendor, subString: "iCab", identity: "iCab" },
		{ string: navigator.vendor, subString: "KDE", identity: "Konqueror" },
		{ string: navigator.userAgent, subString: "Firefox", identity: "Firefox" },
		{ string: navigator.vendor, subString: "Camino", identity: "Camino" },
		{ string: navigator.userAgent, subString: "Netscape", identity: "Netscape" },	// for newer Netscapes (6+)
		{ string: navigator.userAgent, subString: "MSIE", identity: "Explorer", versionSearch: "MSIE" },
		{ string: navigator.userAgent, subString: "Gecko", identity: "Mozilla", versionSearch: "rv" },
		{ string: navigator.userAgent, subString: "Mozilla", identity: "Netscape", versionSearch: "Mozilla" }	// for older Netscapes (4-)
	],

	dataOS: [
		{ string: navigator.platform, subString: "Win", identity: "Windows" },
		{ string: navigator.platform, subString: "Mac", identity: "Mac" },
		{ string: navigator.userAgent, subString: "iPhone", identity: "iPhone/iPod" },
		{ string: navigator.platform, subString: "Linux", identity: "Linux" }
	]
};


/* ----------------------------------------------------
	BASE64 DECODER OBJECT (from: www.webtoolkit.info)
-------------------------------------------------------*/

var base64 = {

	// Private property
	_keyStr : "ABCDEFGHIJKLMNOPQRSTUVWXYZabcdefghijklmnopqrstuvwxyz0123456789+/=",

	// Public method for decoding
	decode : function(input) {
		var output = "";
		var chr1, chr2, chr3;
		var enc1, enc2, enc3, enc4;
		var i = 0;

		input = input.replace(/[^A-Za-z0-9\+\/\=]/g, "");
		while (i < input.length) {
			enc1 = this._keyStr.indexOf(input.charAt(i++));
			enc2 = this._keyStr.indexOf(input.charAt(i++));
			enc3 = this._keyStr.indexOf(input.charAt(i++));
			enc4 = this._keyStr.indexOf(input.charAt(i++));

			chr1 = (enc1 << 2) | (enc2 >> 4);
			chr2 = ((enc2 & 15) << 4) | (enc3 >> 2);
			chr3 = ((enc3 & 3) << 6) | enc4;

			output = output + String.fromCharCode(chr1);

			if (enc3 != 64)
				output = output + String.fromCharCode(chr2);
			if (enc4 != 64)
				output = output + String.fromCharCode(chr3);
		}
		output = base64._utf8_decode(output);
		return output;
	},

	// private method for UTF-8 decoding
	_utf8_decode : function(utftext) {
		var string = "";
		var i = 0;
		var c = c1 = c2 = 0;

		while (i < utftext.length) {
			c = utftext.charCodeAt(i);
			if (c < 128) {
				string += String.fromCharCode(c);
				i++;
			}
			else if ((c > 191) && (c < 224)) {
				c2 = utftext.charCodeAt(i + 1);
				string += String.fromCharCode(((c & 31) << 6) | (c2 & 63));
				i += 2;
			}
			else {
				c2 = utftext.charCodeAt(i + 1);
				c3 = utftext.charCodeAt(i + 2);
				string += String.fromCharCode(((c & 15) << 12) | ((c2 & 63) << 6) | (c3 & 63));
				i += 3;
			}
		}
		return string;
	}
}


/* ----------------------------------------------------
   CLASSES FOR DYNAMICALLY LOAD JAVASCRIPT & CSS
-------------------------------------------------------*/

// JSLoader class for loading external javascripts dynamically. Usage:
// JSLoader.load([
//     { url: "file1.js", id: "scriptid1" },
//     { url: "file2.js", id: "scriptid2" },
//     ...
//     { url: "filex.js", id: "scriptidx" }
// ], callback);

function JSLoader()
{
	var _filequeue = new Array();
	var _filecount = 0;
	var _callback;
	var _scriptcount = 0;
	var _loaded = false;
	var that = this;

	this.load = function(files, callback)
	{
		_filequeue.length = 0;
		_filequeue = files;
		_callback = callback;
		_filecount = 0;
		_loaded = false;

		if (_filequeue.length > 0)
		    this.doLoad(_filequeue[_filecount].url, _filequeue[_filecount].id);
	}

	this.doLoad = function(url, id)
	{
		var oldRef = null;

		if (typeof id == 'undefined' || id.trim() == '') {
			id = 'MyScript' + _scriptcount;
			_scriptcount++;
		} else {
			oldRef = document.getElementById(id);
		}

		var newRef = document.createElement('script');
		newRef.setAttribute('id', id);
		newRef.setAttribute('type', 'text/javascript');
		newRef.setAttribute('src', url);

		// Explorer sucks
		if (browserDetect.browser == 'Explorer') {
			newRef.onreadystatechange = function() {
				if (this.readyState == 'loaded' || this.readyState == 'complete')
					OnLoaded();
			}
		// Anything else
		} else {
			newRef.onload = OnLoaded;
		}

		_loaded = false;

		if (typeof newRef != 'undefined') {
			if (oldRef == null) {
				document.getElementsByTagName('head')[0].appendChild(newRef);
			} else {
				document.getElementsByTagName('head')[0].replaceChild(newRef, oldRef);
			}
		}
	}

	function OnLoaded()
	{
		if (!_loaded) {
			_loaded = true;

			if (_filecount == _filequeue.length - 1) {
				if (_callback != null && typeof _callback == 'function') {  // user callback if defined
					_callback();
				} else if (typeof OnJSLoadComplete == 'function') { // else try default OnJSLoadComplete callback
					OnJSLoadComplete();
				}
			} else {
				_filecount++;
				that.doLoad(_filequeue[_filecount].url, _filequeue[_filecount].id);
			}
		}
	}
}

// CSSLoader class for loading external css stylesheets dynamically. Usage:
// CSSLoader.load([
//     { url: "file1.css", id: "linkid1" },
//     { url: "file2.css", id: "linkid2" },
//     ...
//     { url: "filex.css", id: "linkidx" }
// ]);
// no callback here

function CSSLoader()
{
	var _linkcount = 0;

	this.load = function(files)
	{
		if (files.length > 0) {
		    for (var i = 0; i < files.length; i++) {
			    this.doLoad(files[i].url, files[i].id);
	    	}
		}
	}

	this.doLoad = function(url, id)
	{
		var oldRef = null;

		if (typeof id == 'undefined' || id.trim() == '') {
			id = 'CSSLink' + _linkcount;
			_linkcount++;
		} else {
			oldRef = document.getElementById(id);
		}

		var newRef = document.createElement('link');
		newRef.setAttribute('id', id);
		newRef.setAttribute('rel', 'stylesheet');
		newRef.setAttribute('type', 'text/css');
		newRef.setAttribute('href', url);

		if (typeof newRef != 'undefined') {
			if (oldRef == null) {
				document.getElementsByTagName('head')[0].appendChild(newRef);
			} else {
				document.getElementsByTagName('head')[0].replaceChild(newRef, oldRef);
			}
		}
	}
}


/* ----------------------------------------------------
   PLUGIN OBJECT
------------------------------------------------------*/

function Plugin()
{

	// Public variables

	this.initDone = false;
	this.errorCode = ERROR_NONE;

	// Private variables
	
	var _installTimer;
	var that = this;  // for call 'this' from event handlers

	// Methods ->

	this.init = function(fileName, width, height, bgcolor)
	{
		this.initDone = false;
		this.errorCode = ERROR_NONE;

		// OS requirements
		if (browserDetect.OS != 'Windows' && browserDetect.OS != 'Mac') {
			this.errorCode = ERROR_BADOS;
		    return false;
		}

		// Browser minimum version requirements
		if (!(browserDetect.browser == 'Firefox' && browserDetect.version >= 3.0) &&      // Firefox 3.0
			!(browserDetect.browser == 'Mozilla' && browserDetect.version >= 1.9) &&      // Seamonkey rv 1.9 +(not official)
		    !(browserDetect.browser == 'Explorer' && browserDetect.version >= 6) &&       // IE 6
		    !(browserDetect.browser == 'Safari' && browserDetect.version >= 4) &&         // Safari 4
		    !(browserDetect.browser == 'Opera' && browserDetect.version >= 9) &&          // Opera 9
		    !(browserDetect.browser == 'Chrome' && browserDetect.version >= 4)) {         // Google Chrome 4
			this.errorCode = ERROR_BADBROWSER;
			return false;
		}

		// Plugin installed?
		if (!this.isPluginInstalled()) {
			this.errorCode = ERROR_NOPLUGIN;
			installListener();  // start to listen whether plugin is installed
		    return false;
		} else {
			this.embedPlugin(fileName, width, height, bgcolor);
			this.errorCode = ERROR_NONE;
		    this.initDone = true;
   		    return true;
		}

		this.errorCode = ERROR_UNKNOWN;
	    return false;
	}

	function installListener()
	{
		// Reload page when plugin installed
		navigator.plugins.refresh();
		if (that.isPluginInstalled()) {
			window.location.reload();
		} else {
			_installTimer = setTimeout(installListener, 500);
		}
	}

	this.embedPlugin = function(fileName, width, height, bgcolor)
	{
		var untiyObject = '';
		untiyObject += '<object classid="clsid:444785F1-DE89-4295-863A-D46C3A781394" codebase="http://webplayer.unity3d.com/download_webplayer/UnityWebPlayer.cab#version=2,0,0,0" id="UnityObject" width="' + width + '" height="' + height + '">';
		untiyObject += '<param name="src" value="' + fileName + '" />';
		untiyObject += '<param name="disableContextMenu" value="true" />';
		untiyObject += '<param name="backgroundcolor" value="'+ removeHashmark(bgcolor) + '" />';
		untiyObject += '<param name="bordercolor" value="'+ removeHashmark(bgcolor) + '" />';
		untiyObject += '<param name="textcolor" value="FFFFFF" />';
		untiyObject += '<param name="logoimage" value="' + PATH_SERVICE + '/images/progresslogo.png" />';
		untiyObject += '<param name="progressbarimage" value="' + PATH_SERVICE + '/images/progressbar.png" />';
		untiyObject += '<param name="progressframeimage" value="' + PATH_SERVICE + '/images/progressframe.png" />';
		untiyObject += '<embed type="application/vnd.unity" pluginspage="http://www.unity3d.com/unity-web-player-2.x" id="UnityEmbed" width="' + width + '" height="' + height + '" src="' + fileName + '" disableContextMenu="true" backgroundcolor="'+ removeHashmark(bgcolor) + '" bordercolor="'+ removeHashmark(bgcolor) + '" textcolor="FFFFFF" logoimage="' + PATH_SERVICE + '/images/progresslogo.png" progressbarimage="' + PATH_SERVICE + '/images/progressbar.png" progressframeimage="' + PATH_SERVICE + '/images/progressframe.png" />';
		untiyObject += '</object>';

		var el = element('view3d');
		if (el)
			el.innerHTML = untiyObject;
	}

	this.isPluginInstalled = function()
	{
		var installed = false;
		// Explorer
		if (browserDetect.browser == 'Explorer' && browserDetect.OS == 'Windows') {
			installed = DetectUnityWebPlayerActiveX();  // from detectactivex.vbs
		// Anything else
		} else {
			if (navigator.mimeTypes && navigator.mimeTypes["application/vnd.unity"]) {
				if (navigator.mimeTypes["application/vnd.unity"].enabledPlugin && navigator.plugins && navigator.plugins["Unity Player"]) {
					installed = true;
				}
			}
		}
		return installed;
	}

	this.getEmbeddedObject = function()
	{
		// Explorer
		if (browserDetect.browser == 'Explorer' && browserDetect.OS == 'Windows') {
			return document.getElementById('UnityObject');
		// Safari
		} else if (browserDetect.browser == 'Safari') {
			return document.getElementById('UnityObject');
		// Anything else
		} else {
			return document.getElementById('UnityEmbed');
		}
	}

	this.getPluginVersion = function()
	{
		var r = '';
		if (getEmbeddedObject())
			r = getEmbeddedObject().GetPluginVersion();
	    return r;
	}
}


/* ----------------------------------------------------
   VIEWER OPTIONS
------------------------------------------------------*/

function Options()
{
	// With deault options
	this.controlButtons = '';
	this.backgroundColor = CONTENT3D_BGCOLOR;
	this.fileName = '';
	this.hasSounds = false;
	this.height = CONTENT3D_HEIGHT;
	this.initButton = '';
	this.infoBarVisible = false;
	this.language = LANGUAGE_DEFAULT;
	this.resizeMode = RM_INTERNAL;
	this.sliderVisible = false;
	this.title = '';
	this.titleHTML = '';
	this.useInternals = false;
	this.version = '1.0';
	this.volume = 10;
	this.volumeSliderVisible = false;
	this.width = CONTENT3D_WIDTH;

	// Toolbuttons are off and invisible
	this.toolButtons = {
		"button_sound"      : { visible: false, state: false },
		"button_pan"        : { visible: false, state: false },
		"button_rotating"   : { visible: false, state: false },
		"button_titles"     : { visible: false, state: false },
		"button_anaglyph"   : { visible: false, state: false },
		"button_info"       : { visible: false, state: false },
		"button_help"       : { visible: false, state: false },
		"button_links"      : { visible: false, state: false },
		"button_options"    : { visible: false, state: false },
		"button_fullscreen" : { visible: false, state: false }
	};

	this.settings = {
		"setting_detail_low"    : { state: false },
		"setting_detail_medium" : { state: false },
		"setting_detail_high"   : { state: false }
	};

	this.initLanguages = function()
	{
		// Create language settings form language codes
		if (typeof languages != 'undefined') {
			for (var i = 0; i < languages.length; i++) {
				languages[i] = languages[i].trim().toLowerCase();
				var settingID = 'setting_languages_' + languages[i];
				this.settings[settingID] = { state : languages[i] == this.language };
			}
		}
	}
}


/* ----------------------------------------------------
   VIEWER OBJECT
------------------------------------------------------*/

function Viewer()
{

	// Private objects

	var _plugin = new Plugin();
	var _options = new Options();
	var _slider;

	// Private variables

	var _lastInfoBarText = '';
	var _lastClickedLinkNum = -1;
	var _loadingTimer;
	var _windowState = WS_NORMAL;
	var _doResize = false;
	var _volumeSliderOpened = false;
	var _scrollStep = 3;
	var _scrollTimerLeft = 0;
	var _scrollTimerRight = 0;
	var _scrollButtonsAttached = false;
	var _buttonsWidth = 1;
	var _buttonsContainerWidth = 1;
	var _controlButtonSV = new Array();  // temporarily stores control buttons' states & visibility

	var that = this;       // for call 'this' from event handlers

	// Public variables

	this.initDone = false;   // main init done

	// Methods ->

	// ---- Init ---- //

	this.init = function(options)
	{
		if (typeof OnBeforeInit == 'function')  // OnBeforeInit callback
			OnBeforeInit();

		this.initDone = false;

		_options = options;
        _volumeSliderOpened = _options.volumeSliderVisible;

		if (_options.initButton != '')
			initButton = _options.initButton;  // override initButton

		disableSelection('all');
		this.setPageTitle();

		_plugin.init(_options.fileName, _options.width, _options.height, _options.backgroundColor);

		if (_plugin.initDone) {
			_loadingTimer = setInterval(loadingListener, 200);  // we start to listen loading status
		} else {
		    this.showErrorMessage(_plugin.errorCode);
		    this.initDone = false;
		}

	}

	this.finalInit = function()
	{
		this.initDone = this.initWindow() && this.initToolbar() && this.initVolumeSlider() && this.initSlider() && this.initControlButtons() && this.initSound() && this.initToolButtonStates();

		if (typeof OnInit == 'function')  // OnInit callback
			OnInit();
	}

	function loadingListener()
	{
		if (_ext_Ready) {  // everything has been loaded OK
			clearInterval(_loadingTimer);
			that.finalInit();
		}
		// ...else waiting forever, lol
	}

	this.showErrorMessage = function(errorCode)
	{
		var el = element('view3d');
		if (el)
			el.innerHTML = ERROR_MESSAGES(errorCode);
	}

	this.setPageTitle = function()
	{
		document.title = _options.title;
		var el = element("title");
		if (el) {
			el.innerHTML = _options.titleHTML;
			this.ellipsizeTitle();
		}
	}

	this.ellipsizeTitle = function()
	{
		var el = element('title');
		if (el) {
			var desiredWidth = clampValue1(element('container').offsetWidth - element('toolbuttons').offsetWidth - element('slider').offsetWidth - 65);
			if (desiredWidth < 20) {
				el.style.display = 'none';     // if too small : hide it
			} else {
				el.style.display = 'inline';   // else : ellipsize text (...)
				var text = autoEllipseText(el, _options.titleHTML, desiredWidth);
				el.innerHTML = text;
			}
		}
	}


	// ---- Window ---- //

	this.initWindow = function()
	{
	    window.onresize = this.OnResizeWindow;
		return true;
	}

	var _tempWidth = -1;
	var _tempHeight = -1;

	this.OnResizeWindow = function()
	{
   		// Hack! IE bug: onresize event fires 2-3 times instead of 1, shitf*ck
		if (_tempWidth != document.documentElement.clientWidth || _tempHeight != document.documentElement.clientHeight) {
			if (_options.resizeMode == RM_EXTERNAL) {
				that.resizeContentToWindow();
				that.resizeButtons();
			} else if (_doResize && _options.resizeMode == RM_INTERNAL) {
				if (_windowState == WS_NORMAL) {
				    that.resizeContentToNormal();
					that.resizeButtons();
				} else if (_windowState = WS_MAXIMIZED) {
					that.resizeContentToMaximized();
					that.resizeButtons();
				}
				_doResize = false;
			}
		}
		_tempWidth = document.documentElement.clientWidth;
		_tempHeight = document.documentElement.clientHeight;
	}

	this.maximizeWindow = function()
	{
		window.moveTo(0,0);
		window.resizeTo(screen.availWidth, screen.availHeight);
		_windowState = WS_MAXIMIZED;
	}

	this.restoreWindow = function()
	{

		function resizeToInner(width, height)
		{
			window.resizeTo(screen.availWidth, screen.availHeight);
			var inner = getWindowInnerSize();
			var ox = screen.availWidth - inner.width;
			var oy = screen.availHeight - inner.height;
			window.resizeTo(width + ox, height + oy);
		}

		resizeToInner(WINDOW_WIDTH, WINDOW_HEIGHT);

		var xpos = clampValue0(Math.round((window.screen.width / 2) - (WINDOW_WIDTH / 2)));
		var ypos = clampValue0(Math.round((window.screen.height / 2) - (WINDOW_HEIGHT / 2)));
		window.moveTo(xpos, ypos);
		_windowState = WS_NORMAL;
	}

	this.resizeContentToWindow = function()
	{
	    var inner = getWindowInnerSize();
		inner.width = clampValue1(inner.width);
		inner.height = clampValue1(inner.height);

		element('wrapper').style.width = inner.width + "px";
		element('container').style.width = inner.width + "px";
		element('container').style.height = inner.height + "px";
		element('container').style.marginTop = "0";
		element('container').style.top = "0";
		element('contents').style.width = inner.width + "px";
		element('contents').style.height = clampValue1(inner.height - (TOPBAR_HEIGHT + BOTTOMBAR_HEIGHT)) + "px";
		element('topbar').style.width = inner.width + "px";
		element('bottombar').style.width = inner.width + "px";

		var size = this.calculateContent3DSize();
		size.width = clampValue1(size.width);
		size.height = clampValue1(size.height);

		element('view3d').style.width = size.width + "px";
		element('view3d').style.height = size.height + "px";

		element('sidebar').style.height = clampValue1(size.height - 10) + "px";  // padding -10px;
		element('sidebar').style.display = _options.infoBarVisible ? 'inline' : 'none';

		_options.width = clampValue1(size.width);
		_options.height = clampValue1(size.height);

		this.ellipsizeTitle();
		this.resizeContent3D(size.width, size.height);
	}

	this.resizeContentToMaximized = function() {
	    var inner = getWindowInnerSize();

		element('wrapper').style.width = inner.width + "px";
		element('container').style.width = inner.width + "px";
		element('container').style.height = inner.height + "px";
		element('container').style.marginTop = "0";
		element('container').style.top = "0";
		element('contents').style.width = inner.width + "px";
		element('contents').style.height = (inner.height - (TOPBAR_HEIGHT + BOTTOMBAR_HEIGHT)) + "px";

		var size = this.calculateContent3DSize();

		element('view3d').style.height = size.height + "px";
		element('view3d').style.width = size.width + "px";

		element('sidebar').style.height = (size.height - 10) + "px";  // padding -10px;
		element('sidebar').style.display = _options.infoBarVisible ? 'inline' : 'none';

		_options.width = size.width;
		_options.height = size.height;

		this.ellipsizeTitle();
		this.resizeContent3D(size.width, size.height);
	}

	this.resizeContentToNormal = function() {
		element('wrapper').style.width = WRAPPER_WIDTH + "px";
		element('container').style.width = CONTAINER_WIDTH + "px";
		element('container').style.height = CONTAINER_HEIGHT + "px";
		element('container').style.marginTop = "-" + Math.round(CONTAINER_HEIGHT / 2) + "px";
		element('container').style.top = "50%";

		element('contents').style.width = "100%";
		element('contents').style.height = CONTENTS_HEIGHT + "px";

		var size = this.calculateContent3DSize();

		element('view3d').style.height = size.height + "px";
		element('view3d').style.width = size.width + "px";

		element('sidebar').style.height = (size.height - 10) + "px";  // padding -10px;
		element('sidebar').style.display = _options.infoBarVisible ? 'inline' : 'none';

		_options.width = size.width;
		_options.height = size.height;

		this.ellipsizeTitle();
		this.resizeContent3D(size.width, size.height);
	}

	this.resizeContent3D = function(width, height)
	{
		var el = _plugin.getEmbeddedObject();
		if (el) {
			width = clampValue1(width);
			height = clampValue1(height);
			el.width = width;
			el.height = height;
			el.style.width = width + "px";
			el.style.height = height + "px";
			this.doCommand('cameras.AdjustAnaglyphSize(' + width + ', ' + height + ')');
		}
	}

	this.calculateContent3DSize = function()
	{
		var w = CONTENT3D_WIDTH;                                           // default width
		var h = CONTENT3D_HEIGHT;                                          // default height
		
		if (_options.resizeMode == RM_INTERNAL && _windowState == WS_NORMAL) {
			if (_options.infoBarVisible) {
				w = CONTENT3D_WIDTH - (SIDEBAR_WIDTH + 16 + 2);            // default width - (sidebar width + margins + sidebar border)
			} else {
				w = CONTENT3D_WIDTH;                                       // default width
			}
			h = CONTENT3D_HEIGHT;                                          // default height
		} else {
			var inner = getWindowInnerSize();
			if (_options.infoBarVisible) {
				w = (inner.width - (16 + 2 + 16 + 2)) - SIDEBAR_WIDTH;     // (inner width - (view3d margins + view3d border + sidebar margins + sidebar border)) - sidebar width
			} else {
				w = inner.width - (16 + 2);                                // inner width - (view3d margins +  view3d border)
			}
			h = inner.height - (TOPBAR_HEIGHT + BOTTOMBAR_HEIGHT + 2 + 2); // inner.height - (topbar height + bottombar height + i dunno, lol :) )
		}

		return {width: w, height: h};
	}


	// ---- Toolbar & ToolButtons ---- //

	this.initToolbar = function()
	{
		var el = element("toolbuttons");
		if (el) {
			el.innerHTML = HTML_TOOLBUTTONS();
			for (var i in _options.toolButtons) {
    			this.setToolButtonState(i, _options.toolButtons[i].state);
    			this.setToolButtonVisible(i, _options.toolButtons[i].visible);
   			}
			return true;
		}
		return false;
	}

	this.initToolButtonStates = function()
	{
		if (_options.toolButtons['button_anaglyph'].state)
		    this.setAnaglyph();

		if (_options.toolButtons['button_info'].state) {
			var text = eval(_options.initButton + '_text');
			var isInfoText = this.isInfoBarText(text);
			if (isInfoText) {
				this.setToolButtonVisible('button_info', isInfoText);
				_lastInfoBarText = text;
		    	this.setInfo();
			} else {
				this.setToolButtonState('button_info', false);
			}
	    }

		if (_options.toolButtons['button_help'].state)
		    this.setHelp();

		if (_options.toolButtons['button_links'].state)
		    this.setLinks();

		if (_options.toolButtons['button_options'].state)
		    this.setOptions();

		if (_options.toolButtons['button_fullscreen'].state)
		    this.setFullscreen();

		return true;
	}

	this.setToolButtonVisible = function(buttonID, visible)
	{
	    var el = element(buttonID);
	    if (el && this.getButtonType(buttonID) == BT_TOOLBUTTON) {
			// some special cases
			if (buttonID == 'button_sound' && !_options.hasSounds)
				visible = false;   // force sound button invisible if it has no sounds

			if (buttonID == 'button_sound') {
				if (visible) {
					this.setVolumeSliderVisible(_volumeSliderOpened); // set volumeSlider visible if it's opened before
				} else {
					this.setVolumeSliderVisible(false); // hide volumeSlider when sound button off
				}
			}

			if (buttonID == 'button_links' && (typeof links == 'undefined' || links.length < 1))
				visible = false;   // force links button invisible if no links array

			if (buttonID == 'button_fullscreen' && _options.resizeMode == RM_EXTERNAL)
				visible = false;   // force fullscreen button invisible if resize mode is external

	        el.style.display = visible ? 'block' : 'none';
	        _options.toolButtons[buttonID].visible = visible;
			this.ellipsizeTitle();
		}
	}

	this.getToolButtonVisible = function(buttonID)
	{
		if (this.getButtonType(buttonID) == BT_TOOLBUTTON)
		    return _options.toolButtons[buttonID].visible;
		return false;
	}

	this.setToolButtonState = function(buttonID, state)
	{
	    var el = element(buttonID);
	    if (el && this.getButtonType(buttonID) == BT_TOOLBUTTON) {
			if (buttonID == 'button_sound' && !_options.hasSounds)
				state = false;   // force sound button state false if it has no sounds

			if (state) {
				el.style.backgroundPosition = '0px -' + TOOLBUTTON_HEIGHT + 'px';
			} else {
				el.style.backgroundPosition = '0px 0px';
			}
			_options.toolButtons[buttonID].state = state;
		}
	}

	this.getToolButtonState = function(buttonID)
	{
		if (this.getButtonType(buttonID) == BT_TOOLBUTTON)
		    return _options.toolButtons[buttonID].state;
		return false;
	}


	// ---- ToolButton events ---- //

	/* --- Sound --- */

	this.setSound = function()
	{
		if (this.getToolButtonVisible('button_sound')) {
			this.setVolumeSliderVisible(!_options.volumeSliderVisible);
			_volumeSliderOpened = _options.volumeSliderVisible;
		}
	}

	this.buttonSoundClick = function(sender)
	{
		this.setSound();

		if (typeof buttonSoundClick == 'function')  // buttonSoundClick callback
			buttonSoundClick(sender, _options.toolButtons[sender.id].state);

		internals.update();
	}

    /* --- Pan --- */

	this.setPan = function() {
		if (this.getToolButtonVisible('button_pan')) {
			var state = this.getToolButtonState('button_pan');
			this.doCommand('cameras.SetPanLeftButtonActive(' + state + ')');
		} else {
			this.doCommand('cameras.SetPanLeftButtonActive(' + false + ')');
		}
	}

	this.buttonPanClick = function(sender)
	{
		var b = _options.toolButtons[sender.id].state;
		this.setToolButtonState(sender.id, !b);

		this.setPan();

		if (typeof buttonPanClick == 'function')  // buttonPanClick callback
			buttonPanClick(sender, !b);

		internals.update();
	}

    /* --- Rotating --- */

	this.setRotating = function()
	{
		if (this.getToolButtonVisible('button_rotating')) {
			var state = this.getToolButtonState('button_rotating');
			if (state) {
				this.doCommand('cameras.SetRotatingActive(true)');
			} else {
				this.doCommand('cameras.SetRotatingActive(false)');
			}
		} else {
			this.doCommand('cameras.SetRotatingActive(false)');
		}
	}

	this.buttonRotatingClick = function(sender)
	{
		var b = _options.toolButtons[sender.id].state;
		this.setToolButtonState(sender.id, !b);

		this.setRotating();

		if (typeof buttonRotatingClick == 'function')  // buttonRotatingClick callback
			buttonRotatingClick(sender, !b);

		internals.update();
	}

    /* --- Titles --- */

	this.setTitles = function(buttonClicked)
	{
		if (buttonClicked === undefined)
		    buttonClicked = false;

		if (this.getToolButtonVisible('button_titles')) {
			var state = this.getToolButtonState('button_titles');
			if (state) {
				if (typeof OnShowTitles == 'function')  // OnShowTitles callback
					OnShowTitles(buttonClicked);
			} else {
				if (typeof OnHideTitles == 'function')  // OnHideTitles callback
					OnHideTitles(buttonClicked);
			}
		} else {
			if (typeof OnHideTitles == 'function')  // OnHideTitles callback
				OnHideTitles(buttonClicked);
		}
	}

	this.buttonTitlesClick = function(sender)
	{
		var b = _options.toolButtons[sender.id].state;
		this.setToolButtonState(sender.id, !b);

        this.setTitles(true);

		if (typeof buttonTitlesClick == 'function')  // buttonTitlesClick callback
			buttonTitlesClick(sender, !b);

		internals.update();
	}

    /* --- Anaglyph --- */

	this.setAnaglyph = function()
	{
		if (this.getToolButtonVisible('button_anaglyph')) {
			var state = this.getToolButtonState('button_anaglyph');
			this.doCommand('cameras.SetAnaglyph(' + state + ')');
		}
	}

	this.buttonAnaglyphClick = function(sender)
	{
		var b = _options.toolButtons[sender.id].state;
		this.setToolButtonState(sender.id, !b);

		this.setAnaglyph();

		if (typeof buttonAnaglyphClick == 'function')  // buttonAnaglyphClick callback
			buttonAnaglyphClick(sender, !b);

		internals.update();
	}

    /* --- Info --- */

	this.setInfo = function()
	{
		if (this.getToolButtonVisible('button_info')) {
			var state = this.getToolButtonState('button_info');
			this.setToolButtonState('button_help', false);
			this.setToolButtonState('button_links', false);
			this.setToolButtonState('button_options', false);
			this.setInfoBarText(_lastInfoBarText);
			this.setInfoBarVisible(state);
		}
	}

	this.buttonInfoClick = function(sender)
	{
		var b = _options.toolButtons[sender.id].state;
		this.setToolButtonState(sender.id, !b);

		this.setInfo();

		if (typeof buttonInfoClick == 'function')  // buttonInfoClick callback
			buttonInfoClick(sender, !b);

		internals.update();
	}

    /* --- Help --- */

	this.setHelp = function()
	{
		if (this.getToolButtonVisible('button_help')) {
			var state = this.getToolButtonState('button_help');
			this.setToolButtonState('button_info', false);
			this.setToolButtonState('button_links', false);
			this.setToolButtonState('button_options', false);
			this.setInfoBarText(IT_HELP);
			this.setInfoBarVisible(state);
		}
	}

	this.buttonHelpClick = function(sender)
	{
		var b = _options.toolButtons[sender.id].state;
		this.setToolButtonState(sender.id, !b);

		this.setHelp();

		if (typeof buttonHelpClick == 'function')  // buttonHelpClick callback
			buttonHelpClick(sender, !b);

		internals.update();
	}

    /* --- Links --- */

	this.setLinks = function()
	{
		if (this.getToolButtonVisible('button_links')) {
			var state = this.getToolButtonState('button_links');
			this.setToolButtonState('button_help', false);
			this.setToolButtonState('button_info', false);
			this.setToolButtonState('button_options', false);
			this.setInfoBarText(IT_LINKS);
			this.setInfoBarVisible(state);
		}
	}

	this.buttonLinksClick = function(sender)
	{
		var b = _options.toolButtons[sender.id].state;
		this.setToolButtonState(sender.id, !b);

		this.setLinks();

		if (typeof buttonLinksClick == 'function')  // buttonHelpClick callback
			buttonLinksClick(sender, !b);

		internals.update();
	}

    /* --- Options --- */

	this.setOptions = function()
	{
		if (this.getToolButtonVisible('button_options')) {
			var state = this.getToolButtonState('button_options');
			this.setToolButtonState('button_info', false);
			this.setToolButtonState('button_help', false);
			this.setToolButtonState('button_links', false);
			this.setInfoBarText(IT_OPTIONS);
			this.setInfoBarVisible(state);
		}
	}

	this.buttonOptionsClick = function(sender)
	{
		var b = _options.toolButtons[sender.id].state;
		this.setToolButtonState(sender.id, !b);

		this.setOptions();

		if (typeof buttonOptionsClick == 'function')  // buttonOptionsClick callback
			buttonInfoClick(sender, !b);

		internals.update();
	}

    /* --- Fullscreen --- */

	this.setFullscreen = function()
	{
		if (this.getToolButtonVisible('button_fullscreen')) {
			var state = this.getToolButtonState('button_fullscreen');
			if (state) {
				_doResize = true;
				this.maximizeWindow();
				this.resizeContentToMaximized();
			} else {
				_doResize = true;
				this.restoreWindow();
				this.resizeContentToNormal();
			}
		}
	}

	this.buttonFullscreenClick = function(sender)
	{
		var b = _options.toolButtons[sender.id].state;
		this.setToolButtonState(sender.id, !b);

		this.setFullscreen();

		if (typeof buttonFullscreenClick == 'function')  // buttonFullscreenClick callback
			buttonFullscreenClick(sender, !b);

		internals.update();
	}


	// ----- Slider ----- //
	
	this.initSlider = function()
	{
		var el = element('slider');
		if (el) {
			_slider = new Slider(el, element("slider-input"));
			_slider.setMinimum(SLIDER_MIN);
			_slider.setMaximum(SLIDER_MAX);
			_slider.setValue(SLIDER_VALUE);
			_slider.onchange = this.OnChangeSlider;
			this.setSliderVisible(_options.sliderVisible);
			return true;
		}
		return false;
	}

	this.OnChangeSlider = function()
	{
		that.doSliderZoom(_slider.getValue());
	}

	this.setSliderVisible = function(visible)
	{
		var el = element('slider');
		if (el) {
			el.style.display = visible ? 'inline' : 'none';
			_options.sliderVisible = visible;
			this.ellipsizeTitle();

			// Hack! Set slider value to 0 then the original value to force to redraw(???) that f*cking slider (on Firefox)
			var tempValue = _slider.getValue();
			_slider.onchange = null;
			_slider.setValue(0);
			_slider.setValue(tempValue);
			_slider.onchange = this.OnChangeSlider;
		}
	}

	this.getSliderVisible = function()
	{
		return _options.sliderVisible;
	}

	this.reverseSliderValue = function(value)
	{
		return (_slider.getMaximum() - value) + _slider.getMinimum();
	}

	this.doSliderZoom = function(value)
	{
		this.doCommand('cameras.SetZoomInPercent(cameras.activeCameraIndex, ' + this.reverseSliderValue(value) + ')');
	}
	
	this.setSliderValue = function(value, doEvent)
	{
		if (!doEvent)
			_slider.onchange = null;

		_slider.setValue(this.reverseSliderValue(value));

		if (!doEvent)
			_slider.onchange = this.OnChangeSlider;
	}

	this.resetSlider = function()
	{
		this.setSliderValue(SLIDER_VALUE, false);
	}


	// ---- Volume slider ---- //

	this.initVolumeSlider = function()
	{
		var s = '';
		for (var i = VOLUME_MIN; i <= VOLUME_MAX; i++) {
			s += '<div id="volumestick_' + i + '" class="volumestick" onclick="viewer.volumeStickClick(' + i + ')";></div>'
		}

		var el = element('volumeslider');
		if (el && s != '') {
			el.innerHTML = s;
			if (!_options.hasSounds) {
				this.setVolume(0);
			} else {
				this.setVolume(_options.volume);
			}
			//this.setVolumeSliderVisible(_options.volumeSliderVisible); // don't need this, sets in toolbutton's init (with button_sound)
			return true;
		}
		return false;
	}

	this.setVolumeSliderVisible = function(visible)
	{
		var el = element('volumeslider');
		if (el) {
			el.style.display = visible ? 'inline' : 'none';
			_options.volumeSliderVisible = visible;
			this.ellipsizeTitle();
		}
	}

	this.getVolumeSliderVisible = function()
	{
		return _options.volumeSliderVisible;
	}

	this.setVolume = function(volume)
	{
		if (volume < VOLUME_MIN - 1 || volume > VOLUME_MAX)
			return false;

		for (var i = VOLUME_MIN; i <= VOLUME_MAX; i++) {
			var el = element('volumestick_' + i);
			if (el)
				el.style.backgroundPosition = i <= volume ? '0px -9px' : '0px 0px';
		}
		_options.volume = volume;
		this.setToolButtonState('button_sound', volume > 0);
		this.doCommand('GameObject.Find("/Main Audio Listener").GetComponent(AudioListener).volume = ' + (_options.volume * 0.1) + ';');
	}

	this.volumeStickClick = function(sticknum)
	{
		if (sticknum == 1 && _options.volume > 0)
		    sticknum = 0;
		this.setVolume(sticknum);

		internals.update();
	}


	// ---- Settings ---- //

	this.initSettings = function() {
		for (var settingID in _options.settings) {
		    this.setSettingState(settingID, this.getSettingState(settingID));
		}
		return true;
	}

	this.setSettingState = function(settingID, state)
	{
		if (settingID in _options.settings) {
			var el = element(settingID);
			if (state) {
				if (el) {
					el.style.color = '#ffffff';
					el.style.backgroundPosition = '0px -37px';
				}
				_options.settings[settingID].state = true;
			} else {
				if (el) {
					el.style.color = '#acc0d3';
					el.style.backgroundPosition = '0px 3px';
				}
				_options.settings[settingID].state = false;
			}
		}
	}

	this.getSettingState = function(settingID)
	{
		if (settingID in _options.settings)
			return _options.settings[settingID].state;
		return false;
	}

	this.resetDetailStates = function()
	{
		this.setSettingState('setting_detail_low', false);
		this.setSettingState('setting_detail_medium', false);
		this.setSettingState('setting_detail_high', false);
	}

	this.settingDetailClick = function(sender)
	{
		if (!this.getSettingState(sender.id)) {
			this.resetDetailStates();

			switch (sender.id) {
				case 'setting_detail_low':
					this.doCommand('fpstweak.SetDetail(0)');
				break;
				case 'setting_detail_medium':
					this.doCommand('fpstweak.SetDetail(1)');
				break;
				case 'setting_detail_high':
					this.doCommand('fpstweak.SetDetail(2)');
				break;
			}

			this.setSettingState(sender.id, true);
			internals.update();
		}
	}

	this.resetLanguageStates = function()
	{
	    for (var i = 0; i < languages.length; i++) {
			var settingID = 'setting_languages_' + languages[i];
			this.setSettingState(settingID, false);
	    }
	}

   	this.settingLanguagesClick = function(sender)
	{
		if (!this.getSettingState(sender.id)) {
			this.resetLanguageStates();
			_options.language = sender.id.replace('setting_languages_', '');
			this.translate(_options.language);
			this.doCommand('dispatcher.Translate("' + _options.language + '")');
			this.setSettingState(sender.id, true);
			internals.update();
		}
	}


	// ---- Links ---- //

	this.linksClick = function(linkNum)
	{
		if (typeof links != 'undefined') {
			_lastClickedLinkNum = linkNum;
			internals.update();
			alert('Loading...\n\n' + internals.settings.additional.linkTitle + '\n' + internals.settings.additional.linkURL);
		}
	}
	
	this.getLastClickedLinkNum = function()
	{
		return _lastClickedLinkNum;
	}

	this.getLastClickedLinkTitle = function()
	{
		var r = '';
		if (typeof links != 'undefined' && _lastClickedLinkNum > -1) {
			r = links[_lastClickedLinkNum].title;
		}
		return r;
	}

	this.getLastClickedLinkURL = function()
	{
		var r = '';
		if (typeof links != 'undefined' && _lastClickedLinkNum > -1) {
			r = links[_lastClickedLinkNum].id;
		}
		return r;
	}


	// ---- InfoBar ---- //

	this.setInfoBarVisible = function(visible)
	{
		if ((visible && !_options.infoBarVisible) || (!visible && _options.infoBarVisible)) {
		    _options.infoBarVisible = visible;
			if (_options.resizeMode == RM_EXTERNAL) {
				this.resizeContentToWindow();
				this.resizeButtons();
			} else if (_options.resizeMode == RM_INTERNAL) {
				if (_windowState == WS_NORMAL) {
				    this.resizeContentToNormal();
					this.resizeButtons();
				} else if (_windowState = WS_MAXIMIZED) {
					this.resizeContentToMaximized();
					this.resizeButtons();
				}
			}
		}
	}

	this.getInfoBarVisible = function()
	{
		return _options.infoBarVisible;
	}

	this.isInfoBarText = function(text) {
		return text.trim() != '' && text.indexOf(IT_DEFAULT_TEXT_1) < 0 && text.indexOf(IT_DEFAULT_TEXT_2) < 0 && text.indexOf(IT_DEFAULT_TEXT_3) < 0;
	}

	this.setInfoBarText = function(text)
	{
		var el = element('sidebar');
		if (el) {
			if (text != IT_HELP && text != IT_LINKS && text != IT_OPTIONS && text != IT_CLEAR)
				_lastInfoBarText = text;

			if (text == IT_CLEAR) {
				el.innerHTML = '';
			} else if (text == IT_HELP) {
				el.innerHTML = HTML_HELPTEXT();
			} else if (text == IT_LINKS) {
				el.innerHTML = HTML_LINKSTEXT();
			} else if (text == IT_OPTIONS) {
				el.innerHTML = HTML_OPTIONSTEXT();
				this.initSettings();
			} else {
				// Set infobar invisible when text is an empty string
				if (this.isInfoBarText(text)) {
					this.setToolButtonVisible("button_info", true);
				} else {
					this.setToolButtonState("button_info", false);
					this.setToolButtonVisible("button_info", false);
				}

  				// set infotext only when help, links and options buttons are off
				if (!_options.toolButtons["button_help"].state && !_options.toolButtons["button_links"].state && !_options.toolButtons["button_options"].state) {
					if (!this.isInfoBarText(text))
						this.setInfoBarVisible(false);
					el.innerHTML = text;
				}
			}
			el.scrollTop = 0;
		}
	}


	// ---- ControlButtons ---- //
	
	this.initControlButtons = function()
	{
	    var buttons = element('buttons');
		var bottombar = element('bottombar');
	    if (buttons && bottombar && _options.controlButtons.trim() != '') {
	        buttons.innerHTML = '<div id="buttons-container">' + _options.controlButtons + '</div>'; // add scrollable content
			if (!_scrollButtonsAttached) {
				bottombar.innerHTML = HTML_SCROLLBUTTONS() + bottombar.innerHTML; // add scroll buttons to bottombar (before the buttons div)
				_scrollButtonsAttached = true;
			}
	        this.attachControlButtonGlobalClick();
	        this.resetControlButtons(true);
			this.resizeButtonsContainer();
			this.resizeButtons();
			this.resetScrollPosition();
	        return true;
	    }
	    return false;
	}

	this.attachControlButtonGlobalClick = function()
	{
		// attaches a global event handler to the buttons' onclick event
		var el = element('buttons-container');
		if (el) {
			for (var i = 0, l = el.childNodes.length; i < l; i++) {
				var currentNode = el.childNodes[i];
				if (isStrInClassName(currentNode.className, 'button'))
				    addEvent(currentNode, 'click', this.clickControlButton);
			}
		}
	}

	this.clickControlButton = function(e)
	{
		internals.update();
	}

	this.resetControlButtons = function(resetSwitches)
	{
		var el = element('buttons-container');
		if (el) {
			for (var i = 0, l = el.childNodes.length; i < l; i++) {
				var currentNode = el.childNodes[i];
				if (resetSwitches) {
					if (isStrInClassName(currentNode.className, 'button'))
						this.setControlButtonState(currentNode.id, false);
				} else {
					if (isStrInClassName(currentNode.className, 'button') && !isStrInClassName(currentNode.className, 'switch'))
						this.setControlButtonState(currentNode.id, false);
				}
			}
		}
	}

	this.setControlButtonState = function(buttonID, state)
	{
		var el = element(buttonID);
		if (el && this.getButtonType(buttonID) == BT_CONTROLBUTTON) {
			if (state) {
				if (isStrInClassName(el.className, 'button')) {
					if (!isStrInClassName(el.className, 'switch'))
						this.resetControlButtons(false);
					el.style.backgroundPosition = '0px -' + CONTROLBUTTON_HEIGHT + 'px';
					for (var i = 0, l = el.childNodes.length; i < l; i++) {
						var currentNode = el.childNodes[i];
						if (isStrInClassName(currentNode.className, 'inner'))
							currentNode.style.backgroundPosition = '100% -' + CONTROLBUTTON_HEIGHT + 'px';
					}
				}
			} else {
				if (isStrInClassName(el.className, 'button')) {
					el.style.backgroundPosition = '0px 0px';
					for (var i = 0, l = el.childNodes.length; i < l; i++) {
						var currentNode = el.childNodes[i];
						if (isStrInClassName(currentNode.className, 'inner'))
							currentNode.style.backgroundPosition = '100% 0px';
					}
				}
			}
		}
	}

	this.getControlButtonState = function(buttonID)
	{
	    var state = false;
		var el = element(buttonID);
		if (el && this.getButtonType(buttonID) == BT_CONTROLBUTTON)
			state = el.style.backgroundPosition == '0px 0px';
	    return !state;
	}

	this.setControlButtonVisible = function(buttonID, visible)
	{
	    var el = element(buttonID);
	    if (el && this.getButtonType(buttonID) == BT_CONTROLBUTTON) {
	        el.style.display = visible ? 'block' : 'none';
			this.resizeButtonsContainer();
			this.setScrollButtonsVisible();
			this.setLogoVisible();
        }
	}

	this.getControlButtonVisible = function(buttonID)
	{
		var el = element(buttonID);
		if (el && this.getButtonType(buttonID) == BT_CONTROLBUTTON)
		    return el.offsetWidth > 0;  // el.style.display doesn't work (returns with empty string, why?)
		return false;
	}

	this.storeControlButtonsSV = function()
	{
		// Stores all control buttons' state & visibility in an associative array and keeps it until restore
		_controlButtonSV.length = 0;
		var el = element('buttons-container');
		if (el) {
			for (var i = 0, l = el.childNodes.length; i < l; i++) {
				var currentNode = el.childNodes[i];
				if (isStrInClassName(currentNode.className, 'button')) {
					var SVObject = { state : this.getControlButtonState(currentNode.id), visible : this.getControlButtonVisible(currentNode.id) };
					_controlButtonSV[currentNode.id] = SVObject;
				}
			}
		}
	}

	this.restoreControlButtonsSV = function()
	{
		for (var buttonID in _controlButtonSV) {
			this.setControlButtonState(buttonID, _controlButtonSV[buttonID].state);
			this.setControlButtonVisible(buttonID, _controlButtonSV[buttonID].visible);
		}
		_controlButtonSV.length = 0;
	}

	this.setControlButtonText = function(buttonID, text)
	{
		var el = element(buttonID);
		if (el && this.getButtonType(buttonID) == BT_CONTROLBUTTON) {
			if (isStrInClassName(el.className, 'button')) {
				if (isStrInClassName(el.className, 'fixed')) {
					el.innerHTML = text;
				} else if (isStrInClassName(el.className, 'scaled')) {
					for (var i = 0, l = el.childNodes.length; i < l; i++) {
						var currentNode = el.childNodes[i];
						if (isStrInClassName(currentNode.className, 'inner')) {
							currentNode.innerHTML = text;
						}
					}
				}
			}
			this.resizeButtonsContainer();
			this.setScrollButtonsVisible();
			this.setLogoVisible();
			
		}
	}

	this.getButtonType = function(buttonID)
	{
	    var r = BT_NONE;
	    if (buttonID in _options.toolButtons) {
	        r = BT_TOOLBUTTON;
	    } else {
	        var el = element(buttonID);
	        if (el && isStrInClassName(el.className, 'button'))
	        	r = BT_CONTROLBUTTON;
	    }
	    return r;
	}


	this.calculateButtonsWidth = function()
	{
		width = element('container').offsetWidth - 7 - 8 - 27 - 26; // [buttons div's margins] - [left & right scroll buttons plus their margins]
		_buttonsWidth = width;
		return width;
	}

	this.resizeButtons = function()
	{
		element('buttons').style.width = clampValue1(this.calculateButtonsWidth()) + "px";
		this.setScrollButtonsVisible();
		this.setLogoVisible();
	}

	this.calculateButtonsContainerWidth = function()
	{
		var width = 0;
		var el = element('buttons-container');
		if (el) {
			for (var i = 0, l = el.childNodes.length; i < l; i++) {
				var currentNode = el.childNodes[i];
				if (isStrInClassName(currentNode.className, 'button')) {
					var ow = currentNode.offsetWidth;
					if (ow > 0) {      // offsetWidth > 0 it is visible!
						ow += 6;       // regular buttons' margin
						if (isStrInClassName(currentNode.className, 'separator'))
							ow += 30;  // add separators' margin
						width += ow;
					}
				}
			}
		}
		width += 2;  // Hack! IE9 float bug
		_buttonsContainerWidth = width;
		return width;
	}

	this.resizeButtonsContainer = function()
	{
		element('buttons-container').style.width = clampValue1(this.calculateButtonsContainerWidth()) + "px";
	}


	// ---- Scroll buttons ---- //

	this.scrollButtonLeftClick = function ()
	{
		doScrollLeft();
	}

	this.scrollButtonRightClick = function ()
	{
		doScrollRight();
	}

	function doScrollLeft()
	{
		clearTimeout(_scrollTimerLeft);
		element('buttons').scrollLeft -= _scrollStep;
		_scrollTimerLeft = setTimeout(doScrollLeft, 10);
	}

	function doScrollRight()
	{
		clearTimeout(_scrollTimerRight);
		element('buttons').scrollLeft += _scrollStep;
		_scrollTimerRight = setTimeout(doScrollRight, 10);
	}

	this.resetScrollPosition = function()
	{
		element('buttons').scrollLeft -= 0;
	}

	this.stopScrollTimers = function()
	{
		clearTimeout(_scrollTimerRight);
		clearTimeout(_scrollTimerLeft);
	}

	this.setScrollButtonsVisible = function()
	{
		var visible = _buttonsContainerWidth > _buttonsWidth;
		element('button-scroll-left').style.display = visible ? 'inline' : 'none';
		element('button-scroll-right').style.display = visible ? 'inline' : 'none';
		if (!visible)
			element('buttons').scrollLeft = 0;
	}


	// ---- Logo ---- //
	
	this.setLogoVisible = function() {
		var visible = _buttonsContainerWidth + 75 < _buttonsWidth;
		element('bottombar').style.backgroundPosition = visible ? '100% 9px' : '100% 100px';
	}


	// ---- Sound ---- //

	this.initSound = function()
	{
		if (!_options.hasSounds)
			this.doCommand('sounds.Mute()');
		return true;
	}


	// ---- 3D functions ---- //

	this.doCommand = function(command)
	{
		_plugin.getEmbeddedObject().SendMessage("Main Control", "Dispatch", command);
	}


	// ---- Translate ---- //

	var _lastInitButton = '';

	this.translate = function(languageID)
	{
		_lastInitButton = initButton;
		this.storeControlButtonsSV();

		jsLoader.load([
			{ url: PATH_SERVICE_LANG + '/' + languageID + '/viewer_lang.js', id: 'js_viewer_lang' },
			{ url: '.' + PATH_LANG + '/' + languageID + '/dbinit_lang.js', id: 'js_dbinit_lang' }
		], OnJSLoadComplete);
	}

	function OnJSLoadComplete()
	{
		// Translate service
		if (that.getToolButtonState('button_options'))
			that.setInfoBarText(IT_OPTIONS);
		that.initToolbar();
		that.initVolumeSlider();

		// Translate dbinit
		_options.title = title;
		_options.titleHTML = titleHtml;
		_options.controlButtons = buttons;
		that.setPageTitle(title);
		that.initControlButtons();

		// Restore some old settings
		initButton = _lastInitButton;
		that.restoreControlButtonsSV();
		var textVariable = initButton + '_text';
		if (variableExists(textVariable))
			_lastInfoBarText = eval(textVariable);
		_lastInitButton = '';

		if (typeof OnTranslate == 'function')  // OnTranslate callback
			OnTranslate(_options.language);
	}


	// ---- Misc functions ---- //
	
	this.getOptions = function()
	{
		return _options;
	}
}


/* ----------------------------------------------------
   EXTERNAL EVENT HANDLERS FOR UNITY
   Do not override these functions,
   they are for internal use only!
------------------------------------------------------*/

var _ext_Ready = false;

function OnBeforeReady(anaglyphSupported)
{
	if (viewer) {
		viewer.getOptions().toolButtons["button_anaglyph"].visible = anaglyphSupported;

		// Above html version 3
		if (HTMLVersion.major >= 3) {
			viewer.doCommand('dispatcher.Translate("' + viewer.getOptions().language + '")');  // translating after start
		// Below version 3
		} else {
			// Loads externals after start from the old directory structure
			// Only works in new assets (>1.9.0 or >3.0.0) where "LoadExternalsWithoutTranslate" method exists
			viewer.doCommand('GameObject.Find("/Main Control").SendMessage("LoadExternalsWithoutTranslate", 0, SendMessageOptions.DontRequireReceiver)');
		}
	}
}

function OnReady()
{
	if (!_ext_Ready)
		_ext_Ready = true;
}

function OnCameraZoom(cameraName, distancePercent)
{
	if (viewer)
		viewer.setSliderValue(distancePercent, false);
}

function OnBeforeCameraChange(oldCameraName, newCameraName)
{
	if (viewer) {
		if (oldCameraName != newCameraName) {
			if (typeof OnHideTitles == 'function')  // OnHideTitles callback
				OnHideTitles(false);
		}
	}
}

function OnCameraChange(oldCameraName, newCameraName)
{
	if (viewer) {
		viewer.setPan();
		viewer.setTitles(false);
		viewer.setRotating();
		viewer.resetSlider();
	}
}

function OnAutosetDetail(detail)
{
	if (viewer) {
		switch (detail) {
			case "Low" :
				viewer.resetDetailStates();
				viewer.setSettingState('setting_detail_low', true);
			break;
			case "Medium" :
				viewer.resetDetailStates();
				viewer.setSettingState('setting_detail_medium', true);
			break;
			case "High" :
				viewer.resetDetailStates();
				viewer.setSettingState('setting_detail_high', true);
			break;
			default:
				viewer.resetDetailStates();
				viewer.setSettingState('setting_detail_low', true);
		}
	}
}


/* ----------------------------------------------------
   URL QUERY PARAM FUNCTIONS
------------------------------------------------------*/

function GetURLParams()
{
	var query = window.location.hash.substring(1);
	if (query) {
		query = base64.decode(query);
		var params = query.split('?');
		for (var i = 0; i < params.length; i++) {
			var pos = params[i].indexOf('=');
			if (pos > 0) {
				var key = params[i].substring(0, pos);
				var val = params[i].substring(pos + 1);
				URLParams[key] = val;
			}
		}
	}

	/*
	// Standard parameter query with URL decoding (for testing)

	function URLDecode(str) {
		return unescape(str.replace(/\+/g, " "));
	}
	// Standard method for testing only (change it later)
	var query = window.location.search.substring(1);
	if (query) {
		var params = query.split('&');
		for (var i = 0; i < params.length; i++) {
			var pos = params[i].indexOf('=');
			if (pos > 0) {
				var key = params[i].substring(0, pos);
				var val = params[i].substring(pos + 1);
				URLParams[key] = URLDecode(val);
			}
		}
	}
	*/

}

function URLParamExists(param)
{
	return (typeof URLParams[param] != 'undefined' && URLParams[param].trim() != '');
}


/* ----------------------------------------------------
	INTERNAL SETTINGS
------------------------------------------------------*/

function Internals()
{
	var _settingsElement;
	var that = this;
	
	this.settings;

	this.init = function()
	{
		// Create hidden div for global & local settings (if it doesn't exist)
		_settingsElement = element(HIDDEN_SETTINGS_ID);
		if (!_settingsElement && document.body) {
			var div = document.createElement('div');
			div.id = HIDDEN_SETTINGS_ID;
			document.body.appendChild(div);
			_settingsElement = div;
		}
	}

	this.evalJSON = function(text)
	{
		// Simple JSON parser with security validating (RFC 4627) (from: http://en.wikipedia.org/wiki/JSON)
		return !(/[^,:{}\[\]0-9.\-+Eaeflnr-u \n\r\t]/.test(text.replace(/"(\\.|[^"\\])*"/g, ''))) && eval('(' + text + ')');
	}

	this.update = function()
	{
		if (_settingsElement && viewer) {
			var s = '{';
			var globals = this.getGlobalsText();
			var locals = this.getLocalsText();
            var additional = this.getAdditionalText();

			if (globals != '')
			    s += '"globals":' + globals;
			if (locals != '')
			    s += ',"locals":' + locals;
			if (additional != '')
			    s += ',"additional":' + additional;

			s += '}'

			s = s.replace(/,}/g, '}'); // remove comma from the last items

			_settingsElement.innerHTML = s;
			this.settings = this.evalJSON(s);
		}
	}

	this.getGlobalsText = function()
	{
		var s = '';
		if (viewer) {
			var options = viewer.getOptions();
			s += '{';
			s += '"language":"' + options.language + '",';
			s += '"volume":' + options.volume + ',';
   			s += '}';
		}
		return s;
	}

	this.getLocalsText = function()
	{
		var s = '';
		if (viewer) {
			var options = viewer.getOptions();
			s += '{';
			s += '"initButton":"' + initButton + '",';
			// Toolbuttons
    		for (var i in options.toolButtons)  // get all button
				s += '"' + i + '":' + options.toolButtons[i].state + ',';
			s += '}';
		}
		return s;
	}

	this.getAdditionalText = function()
	{
        var s = '';
		if (viewer) {
			s += '{';
			s += '"linkTitle":"' + viewer.getLastClickedLinkTitle() + '",';
			s += '"linkURL":"' + viewer.getLastClickedLinkURL() + '"';
			s += '}';
		}
		return s;
	}

	this.setSettings = function(text)
	{
		var r = false;
		if (_settingsElement) {
			this.settings = this.evalJSON(text);
			if (this.settings) {
				_settingsElement.innerHTML = text;
				r = true;
			} else {
				r =  false;
			}
		} else {
			return false;
		}
		return r;
	}
}


/* ----------------------------------------------------
   INIT
------------------------------------------------------*/

var _param_internals  = '';
var _param_exec       = '';

function OnJSLoadCompleteAtStart()
{
	initMain();
}

function preinitMain()
{
	HTMLVersion = getHTMLVersion();

	browserDetect.init();
   	internals.init();
	GetURLParams();

	// Handle some url parameter
   	if (URLParamExists('internals'))
		_param_internals = URLParams['internals'];

   	if (URLParamExists('exec'))
		_param_exec = URLParams['exec'];

	// Handles language setting first
	var languageID = LANGUAGE_DEFAULT
	if (_param_internals != '' && internals.setSettings(_param_internals)) {
		if (typeof internals.settings.globals.language != 'undefined' && languageAvailable(internals.settings.globals.language)) {
			languageID = internals.settings.globals.language;
    	} else {
			internals.settings.globals.language = LANGUAGE_DEFAULT;
		}
	}
	languageID = languageID.trim().toLowerCase()

	// Above html version 3 loads external dbinit & viewer language packs
	if (HTMLVersion.major >= 3) {
		jsLoader.load([
			{ url : PATH_SERVICE_LANG + '/' + languageID + '/viewer_lang.js', id : 'js_viewer_lang' },
			{ url : '.' + PATH_LANG + '/' + languageID + '/dbinit_lang.js', id : 'js_dbinit_lang' }
		], OnJSLoadCompleteAtStart);

	// Below html version 3 loads the viewer's default (HU) language pack only
	} else {

		jsLoader.load([
			{ url : PATH_SERVICE_LANG + '/' + LANGUAGE_DEFAULT + '/viewer_lang.js', id : 'js_viewer_lang' }
		], OnJSLoadCompleteAtStart);

	}
}

function initMain()
{
	var options = new Options();

	options.controlButtons = buttons;            // dbinit
	options.backgroundColor = backgroundColor;   // template
	options.fileName = fileName;                 // dbinit
	options.hasSounds = hasSounds;               // dbinit
	options.height = CONTENT3D_HEIGHT;           // auto
	options.initButton = initButton;             // dbinit or internal
	options.infoBarVisible = false;              // auto
	options.language = LANGUAGE_DEFAULT          // internal
	options.resizeMode = RM_EXTERNAL;            // constant
	options.sliderVisible = sliderVisible;       // dbinit
	options.title = title;                       // dbinit
	options.titleHTML = titleHtml;               // dbinit
	options.useInternals = false;                // depends on url parameter exists
	options.version = templateVersion;           // template
	options.volume = 10;                         // internal
	options.volumeSliderVisible = false          // auto
	options.width = CONTENT3D_WIDTH;             // auto

	options.toolButtons["button_sound"].visible = false;             // template
	options.toolButtons["button_pan"].visible = false;               // template
	options.toolButtons["button_rotating"].visible = false;          // template
	options.toolButtons["button_titles"].visible = false;            // template
	options.toolButtons["button_anaglyph"].visible = true;           // depends on render texture & image effects are supported (see OnBeforeReady function)
	options.toolButtons["button_info"].visible = false;              // auto
	options.toolButtons["button_help"].visible = true;               // always true
	options.toolButtons["button_links"].visible = true;              // always true
	options.toolButtons["button_options"].visible = true;            // always true
	options.toolButtons["button_fullscreen"].visible = true;         // always true

	options.toolButtons["button_sound"].state = false;               // template
	options.toolButtons["button_pan"].state = false;                 // template
	options.toolButtons["button_rotating"].state = rotatingState;    // dbinit
	options.toolButtons["button_titles"].state = titlesState;        // dbinit
	options.toolButtons["button_anaglyph"].state = false;            // auto
	options.toolButtons["button_info"].state = false;                // auto
	options.toolButtons["button_help"].state = false;                // auto
	options.toolButtons["button_links"].state = false;               // auto
	options.toolButtons["button_options"].state = false;             // auto
	options.toolButtons["button_fullscreen"].state = false;          // auto

	options.settings["setting_detail_low"].state = false;
	options.settings["setting_detail_medium"].state = false;
	options.settings["setting_detail_high"].state = true;            // default setting is high

	// Override internals if need
	if (_param_internals != '' && internals.settings) {

		// Globals
		with (internals.settings.globals) {
		    if (typeof volume != 'undefined')
				options.volume = volume;

			if (typeof language != 'undefined')
				options.language = language;
   		}

		// Locals
		with (internals.settings.locals) {
		    if (typeof initButton != 'undefined')
				options.initButton = initButton;

			if (typeof button_sound != 'undefined')
				options.toolButtons["button_sound"].state = button_sound;

			if (typeof button_pan != 'undefined')
				options.toolButtons["button_pan"].state = button_pan;

			if (typeof button_rotating != 'undefined')
				options.toolButtons["button_rotating"].state = button_rotating;

			if (typeof button_titles != 'undefined')
				options.toolButtons["button_titles"].state = button_titles;

			if (typeof button_anaglyph != 'undefined')
				options.toolButtons["button_anaglyph"].state = button_anaglyph;

			if (typeof button_info != 'undefined')
				options.toolButtons["button_info"].state = button_info;

			if (typeof button_help != 'undefined')
				options.toolButtons["button_help"].state = button_help;

			if (typeof button_links != 'undefined')
				options.toolButtons["button_links"].state = button_links;

			if (typeof button_options != 'undefined')
				options.toolButtons["button_options"].state = button_options;

			if (typeof button_fullscreen != 'undefined')
				options.toolButtons["button_fullscreen"].state = button_fullscreen;
		}
	}

   	options.initLanguages();  // init language settings

	// Exectues code received from "exec" url parameter
	if (_param_exec != '')
	    eval(_param_exec);

	// Let's go
	viewer.init(options);
}


addEvent(window, 'load', preinitMain);



// That's all folks!


